@@ -1,35 +1,34 @@
 /*----------------------------------------------------------------------------
-    ChucK Concurrent, On-the-fly Audio Programming Language
-      Compiler and Virtual Machine
+  ChucK Concurrent, On-the-fly Audio Programming Language
+    Compiler and Virtual Machine
 
-    Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
-      http://chuck.cs.princeton.edu/
-      http://soundlab.cs.princeton.edu/
+  Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
+    http://chuck.stanford.edu/
+    http://chuck.cs.princeton.edu/
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-    U.S.A.
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+  U.S.A.
 -----------------------------------------------------------------------------*/
 
 //-----------------------------------------------------------------------------
 // file: chuck_type.h
 // desc: chuck type-system / type-checker
 //
-// author: Ge Wang (gewang@cs.princeton.edu)
-//         Perry R. Cook (prc@cs.princeton.edu)
+// author: Ge Wang (ge@ccrma.stanford.edu | gewang@cs.princeton.edu)
 // date: Autumn 2002 - original
-//       Autumn 2004 - rewrite
+//       Autumn 2005 - rewrite
 //-----------------------------------------------------------------------------
 #ifndef __CHUCK_TYPE_H__
 #define __CHUCK_TYPE_H__
@@ -52,7 +51,7 @@ typedef enum {
     te_complex, te_polar, te_string, te_thread, te_shred, te_class,
     te_function, te_object, te_user, te_array, te_null, te_ugen, te_uana, 
     te_event, te_void, te_stdout, te_stderr, te_adc, te_dac, te_bunghole, 
-    te_uanablob, te_io, te_fileio, te_multi
+    te_uanablob, te_io, te_fileio, te_chout, te_cherr, te_multi
 } te_Type;
 
 
@@ -167,7 +166,7 @@ public:
         else if( climb > 0 )
         {
             for( t_CKUINT i = scope.size(); i > 0; i-- )
-            { if( val = (*scope[i-1])[xid] ) break; }
+            { if( ( val = (*scope[i-1])[xid] ) ) break; }
 
             // look in commit buffer
             if( !val && (commit_map.find(xid) != commit_map.end()) )
@@ -202,6 +201,26 @@ public:
             out.push_back( (*iter).second );
         }
     }
+    
+    // get list of top level
+    void get_level( int level, std::vector<Chuck_VM_Object *> & out )
+    {
+        assert( scope.size() >= level );
+        std::map<S_Symbol, Chuck_VM_Object *>::iterator iter;
+        
+        // clear the out
+        out.clear();
+        // get the front of the array
+        std::map<S_Symbol, Chuck_VM_Object *> * m = &commit_map;
+        
+        // go through map
+        for( iter = m->begin(); iter != m->end(); iter++ )
+        {
+            // add
+            out.push_back( (*iter).second );
+        }
+    }
+    
 
 protected:
     std::vector<std::map<S_Symbol, Chuck_VM_Object *> *> scope;
@@ -300,6 +319,8 @@ struct Chuck_Context : public Chuck_VM_Object
 {
     // src_name
     std::string filename;
+    // full filepath (if available) -- added 1.3.0.0
+    std::string full_path;
     // parse tree
     a_Program parse_tree;
     // context namespace
@@ -374,6 +395,7 @@ private:
         global_nspc = global_context.nspc; SAFE_ADD_REF(global_nspc);
         // deprecated stuff
         deprecated.clear(); deprecate_level = 1;
+        user_nspc = NULL;
         // clear
         this->reset();
     }
@@ -383,10 +405,21 @@ protected:
     Chuck_Namespace * global_nspc;
     // global context
     Chuck_Context global_context;
+    // user-global namespace
+    Chuck_Namespace * user_nspc;
 
 public:
     // global namespace
     Chuck_Namespace * global() { return global_nspc; }
+    // global namespace
+    Chuck_Namespace * user()
+    {
+        if(user_nspc == NULL)
+            return global_nspc;
+        else
+            return user_nspc;
+    }
+    
     // namespace stack
     std::vector<Chuck_Namespace *> nspc_stack;
     // expression namespace
@@ -425,17 +458,42 @@ public:
     void reset( )
     {
         // TODO: release stack items?
-        nspc_stack.clear(); nspc_stack.push_back( this->global() );
+        nspc_stack.clear();
+        nspc_stack.push_back( this->global() );
+        if(user_nspc != NULL)
+            nspc_stack.push_back( this->user_nspc );
         // TODO: release stack items?
         class_stack.clear(); class_stack.push_back( NULL );
         // should be at top level
         assert( context == &global_context );
         // assign : TODO: release curr? class_def? func?
         // TODO: need another function, since this is called from constructor
-        curr = this->global(); class_def = NULL; func = NULL;
+        if(user_nspc != NULL)
+            curr = this->user();
+        else
+            curr = this->global();
+        class_def = NULL; func = NULL;
         // make sure this is 0
         class_scope = 0;
     }
+    
+    void load_user_namespace()
+    {
+        // user namespace
+        user_nspc = new Chuck_Namespace;
+        user_nspc->name = "[user]";
+        user_nspc->parent = global_nspc;
+        SAFE_ADD_REF(global_nspc);
+        SAFE_ADD_REF(user_nspc);
+    }
+    
+    void clear_user_namespace()
+    {
+        if(user_nspc) SAFE_RELEASE(user_nspc->parent);
+        SAFE_RELEASE(user_nspc);
+        load_user_namespace();
+        this->reset();
+    }
 
     // top
     Chuck_Namespace * nspc_top( )
@@ -455,6 +513,8 @@ struct Chuck_UGen_Info : public Chuck_VM_Object
 {
     // tick function pointer
     f_tick tick;
+    // multichannel/vector tick function pointer (added 1.3.0.0)
+    f_tickf tickf;
     // pmsg function pointer
     f_pmsg pmsg;
     // number of incoming channels
@@ -471,7 +531,7 @@ struct Chuck_UGen_Info : public Chuck_VM_Object
 
     // constructor
     Chuck_UGen_Info()
-    { tick = NULL; pmsg = NULL; num_ins = num_outs = 1; 
+    { tick = NULL; tickf = NULL; pmsg = NULL; num_ins = num_outs = 1; 
       tock = NULL; num_ins_ana = num_outs_ana = 1; }
 };
 
@@ -516,6 +576,13 @@ struct Chuck_Type : public Chuck_VM_Object
     t_CKBOOL has_constructor;
     // has destructor
     t_CKBOOL has_destructor;
+    // custom allocator
+    f_alloc allocator;
+    
+    // documentation
+    std::string doc;
+    // example files
+    std::vector<std::string> examples;
 
 public:
     // constructor
@@ -527,6 +594,7 @@ public:
         info = NULL; func = NULL; def = NULL; is_copy = FALSE; 
         ugen_info = NULL; is_complete = TRUE; has_constructor = FALSE;
         has_destructor = FALSE;
+        allocator = NULL;
     }
 
     // destructor
@@ -634,6 +702,9 @@ struct Chuck_Value : public Chuck_VM_Object
     Chuck_Func * func_ref;
     // overloads
     t_CKINT func_num_overloads;
+	
+    // documentation
+    std::string doc;
 
     // constructor
     Chuck_Value( Chuck_Type * t, const std::string & n, void * a = NULL,
@@ -687,6 +758,9 @@ struct Chuck_Func : public Chuck_VM_Object
     Chuck_Func * next;
     // for overriding
     Chuck_Value * up;
+	
+    // documentation
+    std::string doc;
 
     // constructor
     Chuck_Func() { def = NULL; code = NULL; is_member = FALSE; vt_index = 0xffffffff; 
@@ -728,6 +802,10 @@ t_CKBOOL type_engine_check_stmt( Chuck_Env * env, a_Stmt stmt );
 t_CKTYPE type_engine_check_exp( Chuck_Env * env, a_Exp exp );
 // add an chuck dll into the env
 t_CKBOOL type_engine_add_dll( Chuck_Env * env, Chuck_DLL * dll, const std::string & nspc );
+// second version: use type_engine functions instead of constructing AST (added 1.3.0.0)
+t_CKBOOL type_engine_add_dll2( Chuck_Env * env, Chuck_DLL * dll, const std::string & dest );
+// import class based on Chuck_DL_Class (added 1.3.2.0)
+t_CKBOOL type_engine_add_class_from_dl( Chuck_Env * env, Chuck_DL_Class * c );
 // type equality
 t_CKBOOL operator ==( const Chuck_Type & lhs, const Chuck_Type & rhs );
 t_CKBOOL operator !=( const Chuck_Type & lhs, const Chuck_Type & rhs );
@@ -737,30 +815,46 @@ t_CKBOOL isa( Chuck_Type * lhs, Chuck_Type * rhs );
 t_CKBOOL isprim( Chuck_Type * type );
 t_CKBOOL isobj( Chuck_Type * type );
 t_CKBOOL isfunc( Chuck_Type * type );
+t_CKBOOL iskindofint( Chuck_Type * type ); // added 1.3.1.0: this includes int + pointers
+t_CKUINT getkindof( Chuck_Type * type ); // added 1.3.1.0: to get the kindof a type
 
 // import
 Chuck_Type * type_engine_import_class_begin( Chuck_Env * env, Chuck_Type * type, 
-                                             Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor = NULL );
+                                             Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor = NULL,
+                                             const char * doc = NULL );
 Chuck_Type * type_engine_import_class_begin( Chuck_Env * env, const char * name, const char * parent,
-                                             Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor = NULL );
+                                             Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor = NULL,
+                                             const char * doc = NULL );
+Chuck_Type * type_engine_import_ugen_begin( Chuck_Env * env, const char * name, const char * parent,
+                                            Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor,
+                                            f_tick tick, f_tickf tickf, f_pmsg pmsg,  // (tickf added 1.3.0.0)
+                                            t_CKUINT num_ins = 0xffffffff, t_CKUINT num_outs = 0xffffffff,
+                                            const char * doc = NULL );
 Chuck_Type * type_engine_import_ugen_begin( Chuck_Env * env, const char * name, const char * parent,
                                             Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor,
                                             f_tick tick, f_pmsg pmsg,
-                                            t_CKUINT num_ins = 0xffffffff, t_CKUINT num_outs = 0xffffffff );
+                                            t_CKUINT num_ins = 0xffffffff, t_CKUINT num_outs = 0xffffffff,
+                                            const char * doc = NULL );
+Chuck_Type * type_engine_import_ugen_begin( Chuck_Env * env, const char * name, const char * parent,
+                                            Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor,
+                                            f_tick tick, f_pmsg pmsg,  const char * doc );
 Chuck_Type * type_engine_import_uana_begin( Chuck_Env * env, const char * name, const char * parent,
                                             Chuck_Namespace * where, f_ctor pre_ctor, f_dtor dtor,
                                             f_tick tick, f_tock tock, f_pmsg pmsg,
                                             t_CKUINT num_ins = 0xffffffff, t_CKUINT num_outs = 0xffffffff,
-                                            t_CKUINT num_ins_ana = 0xffffffff, t_CKUINT num_outs_ana = 0xffffffff );
+                                            t_CKUINT num_ins_ana = 0xffffffff, t_CKUINT num_outs_ana = 0xffffffff,
+                                            const char * doc = NULL );
 t_CKBOOL type_engine_import_mfun( Chuck_Env * env, Chuck_DL_Func * mfun );
 t_CKBOOL type_engine_import_sfun( Chuck_Env * env, Chuck_DL_Func * sfun );
 t_CKUINT type_engine_import_mvar( Chuck_Env * env, const char * type, 
-                                  const char * name, t_CKUINT is_const );
+                                  const char * name, t_CKUINT is_const,
+                                  const char * doc = NULL );
 t_CKBOOL type_engine_import_svar( Chuck_Env * env, const char * type,
                                   const char * name, t_CKUINT is_const,
-                                  t_CKUINT addr );
+                                  t_CKUINT addr, const char * doc = NULL );
 t_CKBOOL type_engine_import_ugen_ctrl( Chuck_Env * env, const char * type, const char * name,
                                        f_ctrl ctrl, t_CKBOOL write, t_CKBOOL read );
+t_CKBOOL type_engine_import_add_ex( Chuck_Env * env, const char * ex );
 t_CKBOOL type_engine_import_class_end( Chuck_Env * env );
 t_CKBOOL type_engine_register_deprecate( Chuck_Env * env, 
                                          const std::string & former, const std::string & latter );
@@ -777,6 +871,12 @@ Chuck_Value * type_engine_find_value( Chuck_Type * type, const std::string & xid
 Chuck_Value * type_engine_find_value( Chuck_Type * type, S_Symbol xid );
 Chuck_Value * type_engine_find_value( Chuck_Env * env, const std::string & xid, t_CKBOOL climb, int linepos = 0 );
 Chuck_Namespace * type_engine_find_nspc( Chuck_Env * env, a_Id_List path );
+/*******************************************************************************
+ * spencer: added this into function to provide the same logic path
+ * for type_engine_check_exp_decl() and ck_add_mvar() when they determine
+ * offsets for mvars -- added 1.3.0.0
+ ******************************************************************************/
+t_CKUINT type_engine_next_offset( t_CKUINT current_offset, Chuck_Type * type );
 // array verify
 t_CKBOOL verify_array( a_Array_Sub array );
 // make array type
@@ -789,6 +889,7 @@ a_Id_List str2list( const std::string & path );
 a_Id_List str2list( const std::string & path, t_CKBOOL & is_array );
 const char * howmuch2str( te_HowMuch how_much );
 t_CKBOOL escape_str( char * str_lit, int linepos );
+t_CKINT str2char( const char * char_lit, int linepos );
 
 // default types
 extern Chuck_Type t_void;
@@ -809,6 +910,8 @@ extern Chuck_Type t_class;
 extern Chuck_Type t_event;
 extern Chuck_Type t_io;
 extern Chuck_Type t_fileio;
+extern Chuck_Type t_chout;
+extern Chuck_Type t_cherr;
 extern Chuck_Type t_ugen;
 extern Chuck_Type t_uana;
 extern Chuck_Type t_uanablob;
